import {
  deleteQuestion,
  updateQuestion,
  getQuestionByContentId,
  clearRuleForAll,
  createResult,
} from "@root/questions/actions";
import { useQuestionsStore } from "@root/questions/store";
import { useCurrentQuiz } from "@root/quizes/hooks";
import { updateRootContentId } from "@root/quizes/actions";

import type { MutableRefObject } from "react";
import type {
  Core,
  CollectionReturnValue,
  PresetLayoutOptions,
} from "cytoscape";
import type {
  AnyTypedQuizQuestion,
  QuestionBranchingRule,
  QuestionBranchingRuleMain,
} from "../../../../model/questionTypes/shared";

type UseRemoveNodeArgs = {
  cyRef: MutableRefObject<Core | null>;
  layoutOptions: PresetLayoutOptions;
  layoutsContainer: MutableRefObject<HTMLDivElement | null>;
  plusesContainer: MutableRefObject<HTMLDivElement | null>;
  crossesContainer: MutableRefObject<HTMLDivElement | null>;
  gearsContainer: MutableRefObject<HTMLDivElement | null>;
};

export const useRemoveNode = ({
  cyRef,
  layoutOptions,
  layoutsContainer,
  plusesContainer,
  crossesContainer,
  gearsContainer,
}: UseRemoveNodeArgs) => {
  const { questions: trashQuestions } = useQuestionsStore();
  const quiz = useCurrentQuiz();

  const removeButtons = (id: string) => {
    layoutsContainer.current
      ?.querySelector(`.popper-layout[data-id='${id}']`)
      ?.remove();
    plusesContainer.current
      ?.querySelector(`.popper-plus[data-id='${id}']`)
      ?.remove();
    crossesContainer.current
      ?.querySelector(`.popper-cross[data-id='${id}']`)
      ?.remove();
    gearsContainer.current
      ?.querySelector(`.popper-gear[data-id='${id}']`)
      ?.remove();
  };

  const clearDataAfterRemoveNode = ({
    targetQuestionContentId,
    parentQuestionContentId,
  }: {
    targetQuestionContentId: string;
    parentQuestionContentId: string;
  }) => {
    updateQuestion(targetQuestionContentId, (question) => {
      question.content.rule.parentId = "";
      question.content.rule.children = [];
      question.content.rule.main = [];
      question.content.rule.default = "";
    });

    //Ищём родителя
    const parentQuestion = getQuestionByContentId(parentQuestionContentId);
    if (parentQuestion.content.rule.children.length === 1) {
      //если у родителя больше нет потомков
      //Делаем результат родителя активным
      const parentResult = trashQuestions.find(
        (q) =>
          q.type === "result" &&
          q.content.rule.parentId === parentQuestionContentId,
      );
      if (parentResult) {
        updateQuestion(parentResult.content.id, (q) => {
          q.content.usage = true;
        });
      } else {
        createResult(quiz?.backendId, parentQuestionContentId);
      }
    }

    //чистим rule родителя
    if (!parentQuestion?.type) {
      return;
    }

    const newChildren = [...parentQuestion.content.rule.children];
    newChildren.splice(
      parentQuestion.content.rule.children.indexOf(targetQuestionContentId),
      1,
    );

    const newRule: QuestionBranchingRule = {
      children: newChildren,
      default:
        parentQuestion.content.rule.default === targetQuestionContentId
          ? ""
          : parentQuestion.content.rule.default,
      //удаляем условия перехода от родителя к этому вопросу,
      main: parentQuestion.content.rule.main.filter(
        (data: QuestionBranchingRuleMain) =>
          data.next !== targetQuestionContentId,
      ),
      parentId: parentQuestion.content.rule.parentId,
    };

    updateQuestion(parentQuestionContentId, (PQ) => {
      PQ.content.rule = newRule;
    });
  };

  const removeNode = (targetNodeContentId: string) => {
    const deleteNodes: string[] = [];
    const deleteEdges: any = [];
    const cy = cyRef?.current;

    const findChildrenToDelete = (node: CollectionReturnValue) => {
      //Узнаём грани, идущие от этой ноды
      cy
        ?.$('edge[source = "' + node.id() + '"]')
        ?.toArray()
        .forEach((edge) => {
          const edgeData = edge.data();

          //записываем id грани для дальнейшего удаления
          deleteEdges.push(edge);
          //ищем ноду на конце грани, записываем её ID для дальнейшего удаления
          const targetNode = cy?.$("#" + edgeData.target);
          deleteNodes.push(targetNode.data().id);
          //вызываем функцию для анализа потомков уже у этой ноды
          findChildrenToDelete(targetNode);
        });
    };

    const elementToDelete = cy?.getElementById(targetNodeContentId);

    if (elementToDelete) {
      findChildrenToDelete(elementToDelete);
    }

    const targetQuestion = getQuestionByContentId(targetNodeContentId);

    if (
      targetQuestion?.type &&
      targetQuestion.content.rule.parentId === "root" &&
      quiz
    ) {
      updateRootContentId(quiz?.id, "");
      updateQuestion(targetNodeContentId, (question) => {
        question.content.rule.parentId = "";
        question.content.rule.main = [];
        question.content.rule.children = [];
        question.content.rule.default = "";
      });
      clearRuleForAll();
    } else {
      const parentQuestionContentId = cy
        ?.$('edge[target = "' + targetNodeContentId + '"]')
        ?.toArray()?.[0]
        ?.data()?.source;
      if (targetNodeContentId && parentQuestionContentId) {
        if (
          quiz &&
          cy?.edges(`[source="${parentQuestionContentId}"]`).length === 0
        ) {
          //createFrontResult(quiz.backendId, parentQuestionContentId);
        }
        clearDataAfterRemoveNode({
          targetQuestionContentId: targetNodeContentId,
          parentQuestionContentId,
        });
        cy
          ?.remove(cy?.$("#" + targetNodeContentId))
          .layout(layoutOptions)
          .run();
      }
    }

    //После всех манипуляций удаляем грани и ноды из CS Чистим rule потомков на беке

    deleteNodes.forEach((nodeId) => {
      //Ноды
      cy?.remove(cy?.$("#" + nodeId));
      removeButtons(nodeId);
      updateQuestion(nodeId, (question) => {
        question.content.rule.parentId = "";
        question.content.rule.main = [];
        question.content.rule.default = "";
        question.content.rule.children = [];
      });
    });

    deleteEdges.forEach((edge: any) => {
      //Грани
      cy?.remove(edge);
    });

    removeButtons(targetNodeContentId);
    cy?.data("changed", true);
    cy?.layout(layoutOptions).run();

    //делаем result всех потомков неактивными
    trashQuestions.forEach((qr) => {
      if (
        qr.type === "result" &&
        (deleteNodes.includes(qr.content.rule.parentId || "") ||
          (targetQuestion?.type &&
            qr.content.rule.parentId === targetQuestion.content.id))
      ) {
        updateQuestion(qr.content.id, (q) => {
          q.content.usage = false;
        });
      }
    });
  };

  return { removeNode };
};
