import { useState, ChangeEvent } from "react";
import {
  Box,
  FormControl,
  FormControlLabel,
  FormLabel,
  Radio,
  RadioGroup,
  Tooltip,
  Typography,
  useMediaQuery,
  useTheme,
} from "@mui/material";

import InfoIcon from "@icons/InfoIcon";

import type { QuestionVariant } from "model/questionTypes/shared";
import type { QuizQuestionVarImg } from "model/questionTypes/varimg";
import RadioCheck from "@ui_kit/RadioCheck";
import RadioIcon from "@ui_kit/RadioIcon";
import { modes } from "@utils/themes/Publication/themePublication";
import { useCurrentQuiz } from "@root/quizes/hooks";

interface Props {
  question: QuizQuestionVarImg;
  widthPreview: number;
}

export default function Varimg({ question, widthPreview }: Props) {
  const [selectedVariantIndex, setSelectedVariantIndex] = useState<number>(-1);
  const theme = useTheme();
  const mode = modes;
  const quiz = useCurrentQuiz();
  const isMobile = useMediaQuery(theme.breakpoints.down(650));
  const handleChange = (event: ChangeEvent<HTMLInputElement>) => {
    setSelectedVariantIndex(
      question.content.variants.findIndex(
        (variant) => variant.answer === event.target.value,
      ),
    );
  };

  const currentVariant: QuestionVariant | undefined =
    question.content.variants[selectedVariantIndex];

  return (
    <Box
      sx={{
        display: "flex",
        flexWrap: "wrap",
        gap: "40px",
      }}
    >
      <FormControl
        className="cancel"
        sx={{
          //maxWidth: "900px",
          width: "100%",
          gap: "25px",
        }}
      >
        <FormLabel
          id="quiz-question-radio-group"
          data-cy="question-title"
          sx={{
            fontSize: "24px",
            fontWeight: 500,
            color: theme.palette.text.primary,
          }}
        >
          {question.title}
        </FormLabel>
        <Box
          sx={{
            display: "flex",
            alignItems: "center",
            flexDirection: widthPreview < 650 ? "column-reverse" : undefined,
            gap: "30px",
          }}
        >
          <RadioGroup
            className="cancel"
            aria-labelledby="quiz-question-radio-group"
            value={currentVariant?.answer ?? ""}
            onChange={handleChange}
            sx={{
              gap: "20px",
              display: "flex",
              width: "100%",
            }}
          >
            {question.content.variants
              .filter(({ answer }) => answer)
              .map((variant, index) => (
                <FormControlLabel
                  key={index}
                  value={variant.answer}
                  data-cy="variant-answer"
                  sx={{
                    margin: 0,
                    borderRadius: "5px",
                    padding: "15px",
                    color: theme.palette.text.primary,
                    border: `1px solid`,
                    borderColor:
                      selectedVariantIndex == index
                        ? theme.palette.primary.main
                        : "#E3E3E3",
                    backgroundColor: mode[quiz.config.theme]
                      ? "white"
                      : theme.palette.background.default,
                    /*display: "flex",*/
                  }}
                  control={
                    <Radio
                      inputProps={{
                        "data-cy": "variant-radio",
                      }}
                      checkedIcon={
                        <RadioCheck color={theme.palette.primary.main} />
                      }
                      icon={<RadioIcon />}
                    />
                  }
                  label={
                    <Box sx={{ display: "flex", alignItems: "center", gap: 2 }}>
                      <Typography sx={{ wordBreak: "break-word" }}>
                        {variant.answer}
                      </Typography>
                      {variant.hints && (
                        <Tooltip title={variant.hints} placement="right">
                          <Box>
                            <InfoIcon />
                          </Box>
                        </Tooltip>
                      )}
                    </Box>
                  }
                />
              ))}
          </RadioGroup>
          <Box
            sx={{
              border: "1px solid #E3E3E3",
              width: widthPreview < 650 ? "300px" : "400px",
              height: widthPreview < 650 ? "300px" : "400px",
              display: "flex",
              justifyContent: "center",
              alignItems: "center",
              borderRadius: "12px",
              marginTop: widthPreview < 650 ? 0 : "50px",
              overflow: "hidden",
            }}
          >
            {currentVariant?.extendedText ? (
              <img
                src={currentVariant.extendedText}
                data-cy="variant-image"
                alt="question variant"
                style={{
                  width: "100%",
                  height: "100%",
                  display: "block",
                  flexGrow: 1,
                  objectFit: "cover",
                }}
              />
            ) : (
              <Typography p={2}>
                {selectedVariantIndex === -1
                  ? widthPreview < 650
                    ? question?.content.replText || "Выберите вариант ниже"
                    : question?.content.replText || "Выберите вариант"
                  : "Картинка отсутствует"}
              </Typography>
            )}
          </Box>
        </Box>
      </FormControl>
    </Box>
  );
}
